<?php

namespace Drupal\entity_pilot\Form;

use Drupal\Core\DependencyInjection\ContainerInjectionInterface;
use Drupal\Core\Entity\ContentEntityConfirmFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Url;
use Drupal\entity_pilot\FlightInterface;

/**
 * Defines a form for moving a departure from ready to queued.
 */
class DepartureQueueForm extends ContentEntityConfirmFormBase implements ContainerInjectionInterface {

  /**
   * The entity being used by this form.
   *
   * @var \Drupal\entity_pilot\DepartureInterface
   */
  protected $entity;

  /**
   * Route name to redirect to.
   *
   * @var string
   */
  protected $redirectRouteName = 'entity_pilot.departure_list';

  /**
   * {@inheritdoc}
   */
  public function getConfirmText() {
    return $this->t('Queue');
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    parent::submitForm($form, $form_state);
    $departure = $this->entity;
    $departure->setNewRevision()
      ->setValidationRequired(FALSE)
      ->setRevisionLog($this->t('Queued by @name', [
        '@name' => $this->currentUser()->getAccountName(),
      ]))
      ->setStatus(FlightInterface::STATUS_QUEUED)
      ->save();
    $this->messenger()->addMessage($this->t('@label %label has been queued.', [
      '%label' => $this->entity->label(),
      '@label' => $this->entity->getEntityType()->getLabel(),
    ]));
    $this->logger('entity_pilot')->notice('@label %label has been queued.', [
      '%label' => $this->entity->label(),
      '@label' => $this->entity->getEntityType()->getLabel(),
    ]);
    $form_state->setRedirect($this->redirectRouteName);
  }

  /**
   * {@inheritdoc}
   */
  public function getQuestion() {
    return $this->t('Are you sure you want to queue %name?', ['%name' => $this->entity->label()]);
  }

  /**
   * {@inheritdoc}
   */
  public function getDescription() {
    return $this->t('Queue the flight for sending to Entity Pilot on next cron-run.');
  }

  /**
   * {@inheritdoc}
   */
  public function getCancelUrl() {
    return new Url($this->redirectRouteName);
  }

}
